@@ -1,6 +1,6 @@
 # A debhelper build system class for handling pkg-js-autopkgtest test.
 #
-# Copyright: 2019, Xavier Guimard <yadd@debian.org>
+# Copyright: Yadd <yadd@debian.org>
 # License: GPL-2+
 
 package Debian::Debhelper::Buildsystem::nodejs;
@@ -56,13 +56,13 @@ sub DESCRIPTION {
 
 # Regexp compiled from dh_nodejs/dh_root_files.excluded
 my $rootFilesIgnored =
-qr/^(?:(?:g(?:ulpfile\.(?:babel\.m?j|(?:t|m?j))|runt(?:file)?\.j)|(?:rollup[\.\-].*config|karma\.conf|.mocharc)\.j)s|b(?:abel\.config\.js(?:on)?|inding\.gyp|ench.*\.js|ower\.json)|c(?:o(?:mponent\.json|ntribute|pying)|hange(?:log|s?)|akefile)|a(?:ut(?:otest\.watchr|hors?)|va\.config\.js|ppveyor\.yml)|j(?:est\.config\.js|sl\.node\.conf|asmine\.json)|.*\.(?:m(?:arkdown|d)|pdf|txt)|t(?:sconfig.*\.json|ests?\.js)|l(?:icen[cs]e.*|erna\.json)|\.(?:babelrc(?:.js)?|.*)|(?:Docker|make)file|package-lock\.json|yarn\.lock|history)$/i;
+qr/^(?:(?:g(?:ulpfile\.(?:babel\.m?j|(?:t|m?j))|runt(?:file)?\.j)|(?:rollup[\.\-].*config|karma\.conf)\.j)s|b(?:abel\.config\.js(?:on)?|inding\.gyp|ench.*\.js|ower\.json)|c(?:o(?:mponent\.json|ntribute|pying)|hange(?:log|s?)|akefile)|a(?:ut(?:otest\.watchr|hors?)|va\.config\.js|ppveyor\.yml)|j(?:est\.config\.js|sl\.node\.conf|asmine\.json)|\.(?:mocharc\.js(?:on)?|babelrc(?:.js)?|.*)|.*\.(?:m(?:arkdown|d)|pdf|txt)|t(?:sconfig.*\.json|ests?\.js)|l(?:icen[cs]e.*|erna\.json)|(?:Docker|make)file|package-lock\.json|yarn\.lock|history)$/i;
 my $filesIgnored =
-qr/^(?:\.(?:(?:wafpickle-|eslint|_).*|npm(?:ignore|rc)|lock-wscript|gitignore)|.*\.(?:c(?:offee|\+\+|cp?)?|orig|def|mk|h)|(?:[jm]|binding.M)akefile|readme(?:\.(?:txt|md))?|package-lock\.json|npm-debug\.log|yarn\.lock)$/i;
+qr/^(?:\.(?:(?:wafpickle-|eslint|_).*|npm(?:ignore|rc)|lock-wscript|gitignore)|.*\.(?:c(?:offee|\+\+|cp?)?|orig|def|mk|h)|(?:[jm]|binding.M)akefile|readme(?:\.(?:txt|md))?|tsconfig\.tsbuildinfo|package-lock\.json|npm-debug\.log|yarn\.lock)$/i;
 
 # Regexp compiled from dh_nodejs/dh_dirs.excluded
 my $dirIgnored =
-qr{^(?:(?:.*/)?(?:\.(?:_.*|git|svn|hg)|.(?:DS_Store|deps)|__[^/]*__|fixtures?|CVS)|(?:archived-packag|node_modul)es|(?:example|test|doc)s?|bench|\..*)(?:/.*)?}i;
+qr{^(?:(?:.*/)?(?:\.(?:_.*|git|svn|hg)|.(?:DS_Store|deps)|__[^/]*__|fixtures?|CVS)|(?:archived-packag|node_modul)es|t(?:ap-snapshots|ests?)|(?:example|doc)s?|bench|\..*)(?:/.*)?}i;
 
 # Extensions that overrides $rootFilesIgnored if set in package.json#files
 my $authorizedExt = qr/\.(?:js)$/;
@@ -313,6 +313,7 @@ sub install_module {
           );
         print "Files to install: " . join( ', ', @files ) . "\n";
     }
+    push @files, autoexcluded() if $dir eq '.';
     my @dest;
     my $mainFile =
       $self->toJs( $dir, $self->pjson($dir)->{main} || "index.js" );
@@ -333,7 +334,7 @@ sub install_module {
         if ( $p =~ s/\!(.*)$// ) {
             push @files, "!$p$1";
         }
-        if ( $p =~ m/[\*|\{]/ ) {
+        if ( $p =~ m/[\*|\{\(]/ ) {
             ( $p, $pattern ) = $self->pattern($p);
         }
         unless ( -e $p ) {
@@ -744,6 +745,10 @@ EOT
             if ( defined( $v = $self->pjson($dir)->{version} ) ) {
                 $tmp .= " (= $v)";
             }
+            else {
+                print STDERR
+                  "# WARNING: missing version for component $tmp ($dir)\n";
+            }
             push @provides, $tmp;
         }
     }
@@ -831,7 +836,9 @@ sub clean {
     my $components = $self->component_list;
     my @dirs       = ( sort keys %$components );
     foreach my $dir ( main_package, @dirs ) {
-        $self->doit_in_sourcedir( 'rm', '-rf', "$dir/node_modules/.cache" );
+        $self->doit_in_sourcedir( 'rm', '-rf', "$dir/node_modules/.cache",
+            "$dir/.nyc_output" );
+        clean_own_stuff($dir);
     }
     clean_external_modules;
     clean_internal_modules;
