@@ -6,14 +6,21 @@ use Dpkg::IPC;
 use File::Path qw(make_path);
 use JSON;
 
+use constant CLEANLIST => [
+    qw(
+      dhnodejsBundle.cjs
+    )
+];
+
 our @EXPORT = qw(
   $OPTS
   clean_build_modules
   clean_external_modules
   clean_internal_modules
   clean_test_modules
+  clean_own_stuff
   component_list
-  root_component_list
+  root_components_list
   link_build_modules
   link_external_modules
   link_internal_modules
@@ -28,10 +35,13 @@ our @EXPORT = qw(
   pjson
   set_debug
   search_bd_in_debian_control
+  autoexcluded
 );
 
 our $DEBUG;
 our $OPTS = { lerna => 1 };
+our @AUTOEXCLUDED;
+our $WANTTS;
 
 sub set_debug {
     $DEBUG = pop @_;
@@ -41,6 +51,10 @@ sub debug {
     print @_, "\n" if $DEBUG;
 }
 
+sub autoexcluded {
+    return map { "!./$_/**/*" } @AUTOEXCLUDED;
+}
+
 sub open_file {
     my $file = pop;
     die "Missing file arg" unless $file;
@@ -62,8 +76,9 @@ sub main_package {
     return ( $main_p ? $main_p : () ) if defined $main_p;
     return '.' unless -e 'debian/nodejs/main';
     my @lines = open_file('debian/nodejs/main');
-    die 'Malformed debian/nodejs/main'              if @lines > 1;
-    print STDERR "Missing main directory $lines[0]" if @lines and !-d $lines[0];
+    die 'Malformed debian/nodejs/main' if @lines > 1;
+    print STDERR "Missing main directory $lines[0]\n"
+      if @lines and !-d $lines[0];
     $main_p = $lines[0] || '';
     return $main_p;
 }
@@ -71,6 +86,7 @@ sub main_package {
 # package.json cache
 sub pjson {
     my $dir = pop;
+    return {} if -f $dir;
     unless ( -e "$dir/package.json" or -e "$dir/package.yaml" ) {
         print STDERR "/!\\ $dir/package.json not found\n";
         $json{$dir} = {};
@@ -102,6 +118,7 @@ sub readLerna {
     my $dir = pop;
     return $lernaFiles{$dir} if $lernaFiles{$dir};
     my $globs = {};
+    my $done;
     if ( -e "$dir/lerna.json" ) {
         debug("$dir/lerna.json found\n");
         my $jsonc;
@@ -109,8 +126,9 @@ sub readLerna {
         my $content = join '', <$jsonc>;
         close $jsonc;
         $globs = JSON::from_json($content);
+        $done++ unless ( $globs->{useWorkspaces} );
     }
-    elsif ( my $tmp = pjson($dir)->{workspaces} ) {
+    if ( !$done and my $tmp = pjson($dir)->{workspaces} ) {
         debug("workspaces field found in package.json\n");
         $globs->{packages} = $tmp;
     }
@@ -119,12 +137,13 @@ sub readLerna {
         $lpkgs = [$lpkgs] unless ref $lpkgs;
         unless ( ref $lpkgs eq 'ARRAY' ) {
             print STDERR
-'Unable to parse lerna.conf#packages and/or package.json#workspaces. Use debian/nodejs/additional_components';
+"Unable to parse lerna.conf#packages and/or package.json#workspaces. Use debian/nodejs/additional_components\n";
             return $lernaFiles{$dir} = {};
         }
         my @pkgs;
         foreach my $entry (@$lpkgs) {
-            push @pkgs, map { s#^\Q$dir/\E##; $_ } glob("$dir/$entry");
+            push @pkgs,
+              map { s#^\Q$dir/\E##; m#^tests?/# ? () : $_ } glob("$dir/$entry");
         }
         $lernaFiles{$dir}->{packages} = \@pkgs;
     }
@@ -133,9 +152,11 @@ sub readLerna {
 
 sub component_list {
     my $lerna = $OPTS->{lerna};
-    my ( @components, %packages );
+    my ( @components, %packages, @toDelete );
     my $addPackage = sub {
         my $dir = shift;
+        next if -f $dir;
+        $dir =~ s#^(?:\.)+/##;
         my $package;
         eval { $package = pjson($dir)->{name} };
         if ( $@ or !$package ) {
@@ -160,8 +181,7 @@ sub component_list {
         debug "Found debian/nodejs/additional_components";
         map {
             if (s/^!//) {
-                my $ref = $_;
-                @components = grep { $_ ne $ref } @components;
+                push @toDelete, $_;
             }
             else {
                 debug 'Adding component(s): ' . join( ', ', glob($_) );
@@ -177,16 +197,22 @@ sub component_list {
                 and my $lernaPackages = readLerna($component)->{packages} )
             {
                 $addPackage->("$component/$_") foreach (@$lernaPackages);
+                push @AUTOEXCLUDED, $component;
             }
             else {
                 $addPackage->($component);
             }
         }
         else {
-            print STDERR "Can't find $component directory in " . `pwd`;
+            print STDERR "Can't find $component directory in " . `pwd` . "\n";
             next;
         }
     }
+    foreach (@toDelete) {
+        my $ref = $_;
+        @components = grep { $_ ne $ref } @components;
+        delete $packages{$ref};
+    }
     return \%packages;
 }
 
@@ -207,6 +233,7 @@ sub ln {
     my $module = pop;
     die "Missing module arg" unless $module;
     my $path = nodepath($module);
+    $module = "\@types/$module" if $WANTTS and $path =~ m#(\@types/.*)?/?$#;
     unless ($path) {
         print STDERR "$module not found in nodejs directories\n";
         return;
@@ -267,6 +294,11 @@ sub clean_test_modules {
     return _cleanMods('debian/tests/test_modules');
 }
 
+sub clean_own_stuff {
+    my $dir = pop;
+    unlink "$dir/$_" foreach ( @{&CLEANLIST} );
+}
+
 # Internal subroutines
 
 sub _link_modules {
@@ -402,6 +434,7 @@ sub _extLinks {
                     nocheck    => 1,
                     wait_child => 1,
                 );
+                debug "Copy $_[0] -> $dir/node_modules/$_[1]";
             }
         );
 
@@ -435,7 +468,7 @@ sub _extLinks {
                         # Search module using nodepath
                         my ($src);
                         spawn(
-                            exec       => [ 'nodepath', $_ ],
+                            exec       => [ 'nodepath', '-B', $_ ],
                             wait_child => 1,
                             to_string  => \$src,
                             nocheck    => 1,
@@ -450,7 +483,7 @@ sub _extLinks {
                                 print STDERR "# Typescript definition "
                                   . "detected, Fallback to main module\n";
                                 spawn(
-                                    exec       => [ 'nodepath', $_ ],
+                                    exec       => [ 'nodepath', '-B', $_ ],
                                     wait_child => 1,
                                     to_string  => \$src,
                                     nocheck    => 1,
@@ -493,14 +526,14 @@ sub _intLinks {
         map {
             unless (/^([\w\-\.\/]+)\s+([\w\-\.\/]+)$/) {
                 print STDERR
-                  "Malformed line $. in debian/nodejs/component_links: $_";
+                  "Malformed line $. in debian/nodejs/component_links: $_\n";
             }
             else {
                 my ( $src, $dest ) = ( $1, $2 );
                 my $success = 1;
                 foreach my $s ( $src, $dest ) {
                     unless ( $components->{$s} ) {
-                        print STDERR "component_links: Unknown component $s";
+                        print STDERR "component_links: Unknown component $s\n";
                         $success = 0;
                     }
                 }
@@ -556,7 +589,7 @@ sub _cleanIntLinks {
         map {
             unless (/^([\w\-\.\/]+)\s+([\w\-\.\/]+)$/) {
                 print STDERR
-                  "Malformed line $. in debian/nodejs/component_links: $_";
+                  "Malformed line $. in debian/nodejs/component_links: $_\n";
             }
             my ( $src, $dest ) = ( $1, $2 );
             debug "unlink $dest/node_modules/$components->{$src}";
@@ -578,7 +611,7 @@ sub nodepath {
     my ($module) = pop;
     my $path;
     spawn(
-        exec       => [ 'nodepath', $module ],
+        exec       => [ 'nodepath', '-B', ( $WANTTS ? ('-t') : () ), $module ],
         to_string  => \$path,
         wait_child => 1,
         nocheck    => 1,
