@@ -1,7 +1,6 @@
-import { minify } from 'terser';
 const pkg = require(process.env.PWD+'/package.json');
 if(!pkg.dependencies) pkg.dependencies={};
-const file = pkg.main ? pkg.main : 'index.cjs';
+const file = process.env.DEST_FILE ? process.env.DEST_FILE : pkg.exports && pkg.exports.require ? pkg.exports.require : 'index.cjs';
 
 const config = {
 	input: process.env.START_FILE,
@@ -17,17 +16,6 @@ const config = {
 		...Object.keys(pkg.dependencies),
 		...require('module').builtinModules,
 	],
-	plugins: [
-		{
-			name: 'terser',
-			renderChunk(code) {
-				return minify(code, {
-					module: true,
-					toplevel: false
-				})
-			}
-		}
-	]
 }
 
 export default config;
